#include <qwidget.h>
#include <qpainter.h>
#include <qsocket.h>
#include <qcopchannel_qws.h>

#include <stdio.h>
#include <stdlib.h>

#include "proxyfb.h"
#include "event.h"

extern QSocket *client;
static proxyevent pevt;

ProxyFB::ProxyFB(QWidget *parent, const char *name) 
	: QWidget(parent, name, WResizeNoErase|WStyle_Customize|WStyle_NoBorderEx)
{
	QCopChannel *qcop = new QCopChannel(QCString("opieproxycmd"), 0, 0);
	connect(qcop, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT(cmd(const QCString&, const QByteArray&)));
}

static void writeEvent() {
	if(client)
		client->writeBlock((char*)&pevt, sizeof(pevt));
}

void ProxyFB::mousePressEvent(QMouseEvent *event) {
	QPoint pos = event->globalPos();
	ButtonState state = event->state();

	printf("Mp x:%d y:%d state:%d\n", pos.x(), pos.y(), state);

	pevt.type = 'M';
	pevt.data.mouse.x = pos.x();
	pevt.data.mouse.y = pos.y();
	pevt.data.mouse.state = state;
	writeEvent();
}

void ProxyFB::mouseReleaseEvent(QMouseEvent *event) {
	QPoint pos = event->globalPos();
	ButtonState state = event->state();

	printf("Mr x:%d y:%d state:%d\n", pos.x(), pos.y(), state);

	pevt.type = 'M';
	pevt.data.mouse.x = pos.x();
	pevt.data.mouse.y = pos.y();
	pevt.data.mouse.state = state;
	writeEvent();
}

void ProxyFB::mouseMoveEvent(QMouseEvent *event) {
	QPoint pos = event->globalPos();
	ButtonState state = event->state();

	printf("Mm x:%d y:%d state:%d\n", pos.x(), pos.y(), state);

	pevt.type = 'M';
	pevt.data.mouse.x = pos.x();
	pevt.data.mouse.y = pos.y();
	pevt.data.mouse.state = state;
	writeEvent();
}

void ProxyFB::keyPressEvent(QKeyEvent *event) {
	printf("Kp %04x\n", event->key());
	event->accept();

	pevt.type = 'K';
	pevt.data.key.code = event->key();
	pevt.data.key.pressed = true;
	writeEvent();
}

void ProxyFB::keyReleaseEvent(QKeyEvent *event) {
	printf("Kr %04x\n", event->key());
	event->accept();

	pevt.type = 'K';
	pevt.data.key.code = event->key();
	pevt.data.key.pressed = false;
	writeEvent();
}

void ProxyFB::focusInEvent(QFocusEvent *event) {
	printf("F in\n");

	pevt.type = 'F';
	pevt.data.focus.enabled = true;
	writeEvent();
}

void ProxyFB::focusOutEvent(QFocusEvent *event) {
	printf("F out\n");

	pevt.type = 'F';
	pevt.data.focus.enabled = false;
	writeEvent();
	hide();
}

void ProxyFB::paintEvent(QPaintEvent *event) {
	printf("paint\n");
	QPainter painter(this);

	painter.fillRect(event->rect(), QColor(255, 255, 0));
}

void ProxyFB::cmd(const QCString & msg, const QByteArray & data) {
	printf("received cmd\n");
	if(msg == "on") {
		printf("enabling...\n");
		showNormal();
		showFullScreen();

		pevt.type = 'F';
		pevt.data.focus.enabled = true;
		writeEvent();
	} else if(msg == "off") {
		printf("disabling...\n");
		hide();

		pevt.type = 'F';
		pevt.data.focus.enabled = false;
		writeEvent();
	} else if(msg == "exit") {
		printf("exiting...\n");
		exit(0);
	}
}
